proto3-suite-0.9.4: A higher-level API to the proto3-wire library
Safe HaskellNone
LanguageHaskell2010

Proto3.Suite.Haskell.Parser

Synopsis

Documentation

data Logger #

parseModule :: Logger -> RealSrcLoc -> StringBuffer -> IO (Maybe (Located (HsModule GhcPs))) Source #

Parses the module with the specified location and content, returning Nothing on parse failure. Errors and warnings are sent to the given Logger, except on GHC 9.0, which would require a DynFlags in order to report messages. Unfortunately, creating a DynFlags would require a directory containing a GHC installation; we do not wish to require that.