| Safe Haskell | None |
|---|---|
| Language | Haskell2010 |
Proto3.Suite.DotProto
Synopsis
- module Proto3.Suite.DotProto.AST
- module Proto3.Suite.DotProto.Parsing
- module Proto3.Suite.DotProto.Rendering
- fieldLikeName :: String -> String
- prefixedEnumFieldName :: String -> String -> String
- typeLikeName :: MonadError CompileError m => String -> m String
Documentation
module Proto3.Suite.DotProto.AST
fieldLikeName :: String -> String Source #
is the casing transformation used to produce record selectors from message fields. If
fieldLikeName fieldfield is prefixed by a span of uppercase characters then that prefix will be lowercased while the remaining string
is left unchanged.
typeLikeName :: MonadError CompileError m => String -> m String Source #
produces either the pascal-cased version of the string typeLikeName xsxs if it begins with an alphabetical
character or underscore - which is replaced with X. A CompileError is emitted if the starting character is
non-alphabetic or if xs == "".