| Safe Haskell | None |
|---|---|
| Language | Haskell2010 |
Proto3.Suite.Haskell.Parser
Synopsis
- data Logger
- initLogger :: IO Logger
- parseModule :: Logger -> RealSrcLoc -> StringBuffer -> IO (Maybe (Located (HsModule GhcPs)))
- renderSDoc :: SDoc -> String
Documentation
initLogger :: IO Logger #
parseModule :: Logger -> RealSrcLoc -> StringBuffer -> IO (Maybe (Located (HsModule GhcPs))) Source #
Parses the module with the specified location and content,
returning Nothing on parse failure. Errors and warnings
are sent to the given Logger, except on GHC 9.0, which
would require a DynFlags in order to report messages.
Unfortunately, creating a DynFlags would require a directory
containing a GHC installation; we do not wish to require that.
renderSDoc :: SDoc -> String Source #